package controllers;

import java.io.IOException;
import java.sql.Timestamp;
import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.FilmCandidateDAO;
import services.FilmDAO;
import services.MeetingDAO;
import services.MemberDAO;
import services.NotificationDAO;
import services.UserDAO;
import utils.Security;
import utils.ValidationUtils;
import dto.Film;
import dto.User;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/createmeeting")
public class CreateMeetingController {

	private @Autowired MeetingDAO meetingDAO;
	private @Autowired MemberDAO memberDAO;
	private @Autowired UserDAO userDAO;
	private @Autowired NotificationDAO notificationDAO;
	private @Autowired FilmDAO filmDAO;
	private @Autowired FilmCandidateDAO filmCandidateDAO;
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/create", method=RequestMethod.POST)
	@ResponseBody
	public int createMeeting(String name, int numberCandidates, boolean inCinema, boolean notSeen, @RequestParam("memberUserSidList[]") List<String> memberSidList, @RequestParam("memberGotItList[]") List<Boolean> memberGotItList, Long date, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(name)
				&& !ValidationUtils.isEmptyOrNull(memberSidList)
				&& !ValidationUtils.isEmptyOrNull(memberGotItList)
				&& (memberSidList.size()==memberGotItList.size())
				&& !ValidationUtils.isEmptyOrNull(hash)){
			if (!inCinema){
				boolean someTrue = false;
				for (boolean gotIt : memberGotItList){
					if (!someTrue){
						if (gotIt){
							someTrue = true;
						}
					}
				}
				if (!someTrue){
					throw new ValidationException("Parameters not valid");
				}
			}
			try{
				if (Security.existHash(hash, userDAO)){
					String userSid = Security.getUsername(hash);
					
					// Checks if there are candidate films
					int count = 0;
					// Film set with custom comparator
					Set<Film> filmList = new TreeSet<Film>(new Comparator<Film>() {
						@Override
						public int compare(Film film1, Film film2) {
							return film1.getSid() - film2.getSid();
						}
					});
					for (String memberSid: memberSidList){
						if (memberGotItList.get(count)){
							filmList.addAll(filmDAO.getCandidateFilms(memberSid, true, notSeen, memberSidList));
						}
						count++;
					}
					if (inCinema){
						filmList.addAll(filmDAO.getInCinemaFilms(notSeen, memberSidList));
					}
					if (filmList.isEmpty()){
						throw new ValidationException("Parameters not valid");
					}
					
					// Creates all
					int meetingSid = meetingDAO.create(name, userSid, numberCandidates, inCinema, notSeen, new Timestamp(date));
					count = 0;
					for (String memberSid: memberSidList){
						memberDAO.create(meetingSid, memberSid, memberGotItList.get(count));
						if (!userSid.equals(memberSid)){
							notificationDAO.createMeeting(userSid, memberSid, meetingSid, false);
						}
						count++;
					}
					for (Film film : filmList){
						filmCandidateDAO.create(meetingSid, film.getSid());
					}
					
					return meetingSid;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
			
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/search", method=RequestMethod.GET)
	@ResponseBody
	public List<User> searchFriends(String words, @RequestParam("users[]") List<String> users, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					String userSid = Security.getUsername(hash);
					List<User> friends = null;
					if (!ValidationUtils.isEmptyOrNull(words)){
						friends = userDAO.getFriendsExcept(words, userSid, users);
					}
					else{
						friends = userDAO.getFriendsExcept(userSid, users);
					}
					// Puts null params that we don't need
					for (User user : friends){
						user.setEmail(null);
						user.setPassword(null);
						user.setRecordDate(null);
						user.setEndedDate(null);
					}
					return friends;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
		
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}
