package controllers;

import java.io.IOException;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.FilmDAO;
import services.UserDAO;
import utils.Security;
import utils.ValidationUtils;
import dto.Film;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/film")
public class FilmController {

	private @Autowired FilmDAO filmDAO;
	private @Autowired UserDAO userDAO;
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/create", method=RequestMethod.POST)
	@ResponseBody
	public Film createFilm(boolean inCinema, String name, String genre,
			String synopsis, Integer premiere, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(name)
				&& !ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					String ownerSid = Security.getUsername(hash);
					int sid = filmDAO.create(ownerSid, inCinema, name, genre, synopsis, premiere);
					// Film returned
					Film film = filmDAO.get(sid);
					// Puts null params that we don't need
					film.setOwnerSid(null);
					film.setRecordDate(null);
					return film;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/{sid}", method=RequestMethod.GET)
	@ResponseBody
	public Film getFilm(@PathVariable("sid") int sid){
		try{
			Film film = filmDAO.get(sid);
			// Puts null params that we don't need
			film.setOwnerSid(null);
			film.setRecordDate(null);
			return film;
		}
		catch (DataAccessException e) {
			System.out.println(e);
			return null;
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/search", method=RequestMethod.GET)
	@ResponseBody
	public List<Film> search(String words, boolean notAdded, String hash) throws IOException, UnauthorizedException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					List<Film> filmList;
					if (notAdded){
						String userSid = Security.getUsername(hash);
						if (!ValidationUtils.isEmptyOrNull(words)){
							filmList = filmDAO.getFilmsNotAddedOrderAlphabetic(userSid, words);
						}
						else{
							filmList = filmDAO.getFilmsNotAddedOrderAlphabetic(userSid);
						}
					}
					else{
						if (!ValidationUtils.isEmptyOrNull(words)){
							filmList = filmDAO.getFilmsOrderAlphabetic(words);
						}
						else{
							filmList = filmDAO.getFilmsOrderAlphabetic();
						}
					}
					// Puts null params that we don't need
					for (Film film : filmList){
						film.setOwnerSid(null);
						film.setRecordDate(null);
						film.setInCinema(null);
						film.setPremiere(null);
						film.setSynopsis(null);
						film.setGenre(null);
					}
					return filmList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
		
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}
