package controllers;

import java.io.IOException;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.FilmDAO;
import services.MeetingDAO;
import services.UserDAO;
import utils.Security;
import utils.ValidationUtils;
import dto.Film;
import dto.Meeting;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/home")
public class HomeController {

	private @Autowired FilmDAO filmDAO;
	private @Autowired UserDAO userDAO;
	private @Autowired MeetingDAO meetingDAO;
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/last7films", method=RequestMethod.GET)
	@ResponseBody
	public List<Film> lastSevenFilms(String hash) throws IOException, UnauthorizedException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					List<Film> filmList = filmDAO.getLastFilmsOrderRecordDate(7);
					// Puts null params that we don't need
					for (Film film : filmList){
						film.setOwnerSid(null);
						film.setRecordDate(null);
						film.setInCinema(null);
						film.setPremiere(null);
						film.setSynopsis(null);
						film.setGenre(null);
					}
					return filmList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/nextmeetings", method=RequestMethod.GET)
	@ResponseBody
	public List<Meeting> nextMeetings(String userSid, String hash) throws IOException, UnauthorizedException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(userSid) 
				&& !ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					List<Meeting> meetingList = meetingDAO.getNextMeetings(userSid);
					// Puts null params that we don't need
					for (Meeting meeting : meetingList){
						meeting.setUserSid(null);
						meeting.setInCinema(null);
						meeting.setNotSeen(null);
						meeting.setNumberCandidates(null);
						meeting.setVoted(null);
					}
					return meetingList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
		
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}
