package controllers;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.FilmCandidateDAO;
import services.FilmDAO;
import services.MeetingDAO;
import services.MemberDAO;
import services.UserDAO;
import utils.Security;
import utils.ValidationUtils;
import dto.Film;
import dto.Meeting;
import dto.Member;
import dto.Pair;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/meetings")
public class MeetingsController {

	private @Autowired MeetingDAO meetingDAO;
	private @Autowired FilmDAO filmDAO;
	private @Autowired UserDAO userDAO;
	private @Autowired MemberDAO memberDAO;
	private @Autowired FilmCandidateDAO filmcandidateDAO;
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/get/{sid}", method=RequestMethod.GET)
	@ResponseBody
	public Meeting getMeeting(@PathVariable("sid") int meetingSid, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					Meeting meeting = meetingDAO.get(meetingSid);
					String userSid = Security.getUsername(hash);
					boolean alreadyVoted = memberDAO.get(meetingSid, userSid).isVoted();
					// Puts null params that we don't need
					if (alreadyVoted){
						meeting.setInCinema(null);
						meeting.setNotSeen(null);
						meeting.setNumberCandidates(null);
					}
					meeting.setUserSid(null);
					return meeting;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/me", method=RequestMethod.GET)
	@ResponseBody
	public List<Meeting> myMeetings(String userSid, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(userSid)
				&& !ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					List<Meeting> meetingList = meetingDAO.myMeetings(userSid);
					// Puts null params that we don't need
					for (Meeting meeting : meetingList){
						meeting.setUserSid(null);
						meeting.setInCinema(null);
						meeting.setNotSeen(null);
						meeting.setNumberCandidates(null);
					}
					return meetingList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/allmember", method=RequestMethod.GET)
	@ResponseBody
	public List<Member> getAllMembers(int meetingSid, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					List<Member> memberList = memberDAO.getAll(meetingSid);
					// Puts null params that we don't need
					for (Member member : memberList){
						member.setMeetingSid(null);
					}
					return memberList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/rankingfilms", method=RequestMethod.GET)
	@ResponseBody
	public List<Pair<Film, Integer>> getRankingFilms(int meetingSid, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					List<Pair<Film, Integer>> pairList = new ArrayList<Pair<Film, Integer>>();
					List<Film> filmList = filmDAO.getRankingFilms(meetingSid);
					// Puts null params that we don't need
					for (Film film : filmList){
						film.setOwnerSid(null);
						film.setPremiere(null);
						film.setRecordDate(null);
						film.setSynopsis(null);
						film.setInCinema(null);
						
						Pair<Film, Integer> pair = new Pair<Film, Integer>();
						pair.setA(film);
						pair.setB(filmcandidateDAO.getPoints(meetingSid, film.getSid()));
						pairList.add(pair);
					}
					return pairList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/candidates", method=RequestMethod.GET)
	@ResponseBody
	public List<Film> getCandidates(String words, int meetingSid, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					List<Film> filmList = null;
					if (ValidationUtils.isEmptyOrNull(words)){
						filmList = filmDAO.getAllCandidates(meetingSid);
					}
					else{
						filmList = filmDAO.getSearchCandidates(words, meetingSid);
					}
					// Puts null params that we don't need
					for (Film film : filmList){
						film.setOwnerSid(null);
						film.setPremiere(null);
						film.setRecordDate(null);
						film.setSynopsis(null);
						film.setInCinema(null);
					}
					return filmList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
		
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}
