package controllers;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.MeetingDAO;
import services.NotificationDAO;
import services.UserDAO;
import utils.Security;
import utils.ValidationUtils;
import dto.Notification;
import dto.Pair;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/notifications")
public class NotificationsController {

	private @Autowired UserDAO userDAO;
	private @Autowired NotificationDAO notificationDAO;
	private @Autowired MeetingDAO meetingDAO;
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/get", method=RequestMethod.GET)
	@ResponseBody
	public List<Pair<Notification,String>> getNotifications(String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					List<Pair<Notification,String>> pairList = new ArrayList<Pair<Notification,String>>();
					String userSid = Security.getUsername(hash);
					List<Notification> notificationList = notificationDAO.getNotifications(userSid);
					for (Notification notification : notificationList){
						Pair<Notification, String> pair = new Pair<Notification, String>();
						pair.setA(notification);
						if (notification.getMeetingSid() != null){
							pair.setB(meetingDAO.get(notification.getMeetingSid()).getName());
						}
						pairList.add(pair);
					}
					return pairList;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/seen/{sid}", method=RequestMethod.POST)
	@ResponseBody
	public void seen(@PathVariable("sid") int sid, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					notificationDAO.seen(sid);
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/news", method=RequestMethod.GET)
	@ResponseBody
	public boolean news(String userSid, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					return notificationDAO.news(userSid);
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
		
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}
