package services;

import java.util.List;

import org.springframework.dao.DataAccessException;

import dto.Film;

public interface FilmDAO {
	
	/**
	 * Creates a Film
	 * @param ownerSid String
	 * @param inCinema boolean
	 * @param name String
	 * @param genre String
	 * @param synopsis String
	 * @param premiere Integer
	 * @return int sid returned
	 * @throws DataAccessException
	 */
	int create(String ownerSid, boolean inCinema, String name, String genre, String synopsis, Integer premiere) throws DataAccessException;
	
	/**
	 * Gets a specific Film
	 * @param sid String identificator of the Film
	 * @return Film returned
	 * @throws DataAccessException
	 */
	Film get(int sid) throws DataAccessException;
	
	/**
	 * Gets user films order by record date of his library
	 * @param userSid String
	 * @return List<Film> films returned
	 * @throws DataAccessException
	 */
	List<Film> getUserFilmsOrderRecordDate(String userSid) throws DataAccessException;
	
	/**
	 * Gets filtered user films order by record date of his library
	 * @param userSid String
	 * @param gotIt boolean
	 * @param seeWannaSee Boolean
	 * @return List<Film> films returned
	 * @throws DataAccessException
	 */
	List<Film> getUserFilmsWithFiltersOrderRecordDate(String userSid, boolean gotIt, Boolean seeWannaSee) throws DataAccessException;
	
	/**
	 * Gets films ordered alphabetically
	 * @return List<Film> films returned
	 * @throws DataAccessException
	 */
	List<Film> getFilmsOrderAlphabetic() throws DataAccessException;
	
	/**
	 * Gets films ordered alphabetically between words
	 * @param words String
	 * @return List<Film> films returned
	 * @throws DataAccessException
	 */
	List<Film> getFilmsOrderAlphabetic(String words) throws DataAccessException;
	
	/**
	 * Gets films ordered alphabetically and not added for the user yet
	 * @param userSid String
	 * @return List<Film> films returned
	 * @throws DataAccessException
	 */
	List<Film> getFilmsNotAddedOrderAlphabetic(String userSid) throws DataAccessException;
	
	/**
	 * Gets films ordered alphabetically between words and not added for the user yet
	 * @param userSid String
	 * @param words String
	 * @return List<Film> films returned
	 * @throws DataAccessException
	 */
	List<Film> getFilmsNotAddedOrderAlphabetic(String userSid, String words) throws DataAccessException;
	
	/**
	 * Gets films order by record date
	 * @param numResults int maxium number of results
	 * @return List<Film> films returned
	 * @throws DataAccessException
	 */
	List<Film> getLastFilmsOrderRecordDate(int numResults) throws DataAccessException;
	
	/**
	 * Gets films that are in the cinema
	 * @param notSeen boolean
	 * @param memberSidList List<String> 
	 * @return List<Film> returned
	 * @throws DataAccessException
	 */
	List<Film> getInCinemaFilms(boolean notSeen, List<String> memberSidList) throws DataAccessException;
	
	/**
	 * Gets candidate films of a member
	 * @param userSid 
	 * @param notSeen boolean
	 * @param memberSidList List<String>
	 * @return List<Film> returned
	 * @throws DataAccessException
	 */
	List<Film> getCandidateFilms(String userSid, boolean gotIt, boolean notSeen, List<String> memberSidList) throws DataAccessException;
	
	/**
	 * Gets all candidates
	 * @param meetingSid int
	 * @return List<Film> returned
	 * @throws DataAccessException
	 */
	List<Film> getAllCandidates(int meetingSid) throws DataAccessException;
	
	/**
	 * Searches by candidates
	 * @param words String
	 * @param meetingSid int
	 * @return List<Film> returned
	 * @throws DataAccessException
	 */
	List<Film> getSearchCandidates(String words, int meetingSid) throws DataAccessException;
	
	/**
	 * Gets all films for a meeting with points > 0
	 * @param meetingSid int
	 * @return List<Film> returned
	 * @throws DataAccessException
	 */
	List<Film> getRankingFilms(int meetingSid) throws DataAccessException;
}
