package services.impl;


import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import services.FilmDAO;
import dto.Film;
 
@Repository
public class FilmDAOImpl implements FilmDAO{
	
	private @Autowired GeneralDAO generalDAO;
	
	@Transactional(readOnly = false)
	public int create(final String ownerSid, final boolean inCinema, final String name, final String genre, final String synopsis, final Integer premiere) throws DataAccessException{
		try{
			final String query = "INSERT INTO film (owner_sid, in_cinema, name, genre, synopsis, premiere, record_date) VALUES (?, ?, ?, ?, ?, ?, ?)";
			final Timestamp recordDate = new Timestamp(System.currentTimeMillis());
			
			KeyHolder holder = new GeneratedKeyHolder();
			generalDAO.getJdbcTemplate().update(new PreparedStatementCreator() {           

                @Override
                public PreparedStatement createPreparedStatement(Connection connection)
                        throws SQLException {
                    PreparedStatement ps = connection.prepareStatement(query, Statement.RETURN_GENERATED_KEYS);
                    ps.setString(1, ownerSid);
                    ps.setBoolean(2, inCinema);
                    ps.setString(3, name);
                    ps.setString(4, genre);
                    ps.setString(5, synopsis);
                    ps.setObject(6, premiere);
                    ps.setTimestamp(7, recordDate);
                    return ps;
                }
            }, holder);
			
			// Sid returned
			return holder.getKey().intValue();
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public Film get(int sid) throws DataAccessException{
		try{
			String query = "SELECT * FROM film WHERE sid = ?";
			Object[] params = new Object[]{
				sid	
			};
			Film film = generalDAO.getJdbcTemplate().queryForObject(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return film;
		}
		catch (EmptyResultDataAccessException e) {
			return null;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}	
	
	
	@Transactional(readOnly = true)
	public List<Film> getUserFilmsOrderRecordDate(String userSid) throws DataAccessException {
		try{
			String query = "SELECT * FROM film INNER JOIN filmlibrary flibrary ON flibrary.film_sid=sid" +
					" WHERE flibrary.user_sid=? ORDER BY flibrary.record_date DESC";
			Object[] params = new Object[]{
				userSid	
			};
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = true)
	public List<Film> getUserFilmsWithFiltersOrderRecordDate(String userSid, boolean gotIt, Boolean seeWannaSee) throws DataAccessException {
		try{
			// Checks if the operator seeWannaSee is not null
			String operatorSeeWannaSee = "";
			if (seeWannaSee != null){
				operatorSeeWannaSee = " AND flibrary.see_wannasee = " + seeWannaSee;
			}
			String query = "SELECT * FROM film INNER JOIN filmlibrary flibrary ON flibrary.film_sid=sid" +
				" WHERE flibrary.user_sid=? AND flibrary.got_it=?" + operatorSeeWannaSee + " ORDER BY flibrary.record_date DESC";
			Object[] params = new Object[]{
				userSid, gotIt
			};
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Film> getFilmsOrderAlphabetic() throws DataAccessException {
		try{
			String query = "SELECT * FROM film ORDER BY name ASC";
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Film> getFilmsOrderAlphabetic(String words) throws DataAccessException {
		try{
			String query = "SELECT * FROM film WHERE name LIKE ? ORDER BY name ASC";
			Object[] params = new Object[]{
				"%"+words+"%"
			};
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Film> getFilmsNotAddedOrderAlphabetic(String userSid) throws DataAccessException {
		try{
			String query = "SELECT * FROM film WHERE NOT EXISTS (SELECT * FROM filmlibrary flibrary WHERE flibrary.film_sid=sid AND flibrary.user_sid=? AND flibrary.got_it=true)" +
					" ORDER BY name ASC";
			Object[] params = new Object[]{
				userSid	
			};
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Film> getFilmsNotAddedOrderAlphabetic(String userSid, String words) throws DataAccessException {
		try{
			String query = "SELECT * FROM film WHERE NOT EXISTS (SELECT * FROM filmlibrary flibrary WHERE flibrary.film_sid=sid AND flibrary.user_sid=? AND flibrary.got_it=true)" +
					" AND name LIKE ? ORDER BY name ASC";
			Object[] params = new Object[]{
				userSid, "%"+words+"%"	
			};
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Film> getLastFilmsOrderRecordDate(int numResults) throws DataAccessException {
		try{
			String query = "SELECT * FROM film ORDER BY record_date DESC LIMIT 0, ?";
			Object[] params = new Object[]{
				numResults
			};
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Film> getInCinemaFilms(boolean notSeen, List<String> memberSidList) throws DataAccessException {
		try{
			String notSeenQuery = "";
			if (notSeen){
				int count = 1;
				int size = memberSidList.size();
				String inMemberSid = "(";
				for (String memberSid : memberSidList){
					inMemberSid = inMemberSid + "'" + memberSid + "'";
					if (count != size){
						inMemberSid = inMemberSid + ", ";
					}
					count++;
				}
				inMemberSid = inMemberSid + ")";
				
				notSeenQuery = " AND NOT EXISTS (SELECT * FROM member INNER JOIN filmlibrary flibrary ON flibrary.user_sid=member.user_sid " +
							"WHERE film.sid=flibrary.film_sid AND member.user_sid IN "+inMemberSid+" AND flibrary.see_wannasee=true)";
			}
			String query = "SELECT * FROM film WHERE in_cinema=true"+notSeenQuery;
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = true)
	public List<Film> getCandidateFilms(String userSid, boolean gotIt, boolean notSeen, List<String> memberSidList)
			throws DataAccessException {
		try{
			String notSeenQuery = "";
			if (notSeen){
				int count = 1;
				int size = memberSidList.size();
				String inMemberSid = "(";
				for (String memberSid : memberSidList){
					inMemberSid = inMemberSid + "'" + memberSid + "'";
					if (count != size){
						inMemberSid = inMemberSid + ", ";
					}
					count++;
				}
				inMemberSid = inMemberSid + ")";
				
				notSeenQuery = " AND NOT EXISTS (SELECT * FROM user u INNER JOIN filmlibrary flibrary ON flibrary.user_sid=u.username " +
							"WHERE film.sid=flibrary.film_sid AND u.username IN "+inMemberSid+" AND flibrary.see_wannasee=true)";
			}
			
			List<Film> filmList = new ArrayList<Film>();
			String query = "";
			if (gotIt){
				query = "SELECT * FROM film WHERE EXISTS (SELECT * FROM user u INNER JOIN filmlibrary flibrary ON flibrary.user_sid=u.username WHERE" +
						" u.username=? AND flibrary.film_sid=film.sid AND flibrary.got_it=true)" + notSeenQuery;
				Object[] params = new Object[]{
					userSid	
				};
				filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			}
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Film> getAllCandidates(int meetingSid) throws DataAccessException{
		try{
			String query = "SELECT * FROM film INNER JOIN filmcandidate fcandidate " +
					"ON fcandidate.film_sid=sid WHERE fcandidate.meeting_sid=? ORDER BY name ASC";
			Object[] params = new Object[]{
				meetingSid
			};
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Film> getSearchCandidates(String words, int meetingSid) throws DataAccessException{
		try{
			String query = "SELECT * FROM film INNER JOIN filmcandidate fcandidate " +
					"ON fcandidate.film_sid=sid WHERE fcandidate.meeting_sid=? AND name LIKE ? ORDER BY name ASC";
			Object[] params = new Object[]{
				meetingSid, "%"+words+"%"
			};
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Film> getRankingFilms(int meetingSid) throws DataAccessException {
		try{
			String query = "SELECT * FROM film INNER JOIN filmcandidate fcandidate ON fcandidate.film_sid=film.sid WHERE fcandidate.points>0 AND EXISTS " +
					"(SELECT * FROM meeting WHERE meeting.sid = ? AND meeting.sid = fcandidate.meeting_sid)" +
					" ORDER BY points DESC";
			Object[] params = new Object[]{
				meetingSid	
			};
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	
	@Transactional(readOnly = true)
	public List<Film> getAll() throws DataAccessException{
		try{
			String query = "SELECT * FROM film";
			List<Film> filmList = generalDAO.getJdbcTemplate().query(query, new BeanPropertyRowMapper<Film>(Film.class));
			return filmList;
		}
		catch (DataAccessException e) {
			throw e;
		}
		
	}
	
	@Transactional(readOnly = true)
	public String getName(int sid) throws DataAccessException{
		try{
			String query = "SELECT name FROM film where sid = ?";
			Object[] params = new Object[]{
				sid	
			};
			String name = generalDAO.getJdbcTemplate().queryForObject(query, params, String.class);
			return name;
		}
		catch (EmptyResultDataAccessException e) {
			return null;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<String> getAllName() throws DataAccessException{
		try{
			String query = "SELECT name FROM film";
			List<String> nameList = generalDAO.getJdbcTemplate().queryForList(query, String.class);
			return nameList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
}